@@ -84,10 +84,10 @@ def test_select_columns(df):
 
 
 def test_filter(df):
-    df = df.select(
+    df = df.filter(column("a") > literal(2)).select(
         column("a") + column("b"),
         column("a") - column("b"),
-    ).filter(column("a") > literal(2))
+    )
 
     # execute and collect the first (and only) batch
     result = df.collect()[0]
@@ -370,6 +370,9 @@ def test_execution_plan(aggregate_df):
 
     assert expected == plan.display()
 
+    # Check the number of partitions is as expected.
+    assert type(plan.partition_count) is int
+
     expected = (
         "ProjectionExec: expr=[c1@0 as c1, SUM(test.c2)@1 as SUM(test.c2)]\n"
         "  Aggregate: groupBy=[[test.c1]], aggr=[[SUM(test.c2)]]\n"
@@ -611,7 +614,6 @@ def test_to_pydict(df):
 
 
 def test_describe(df):
-
     # Calculate statistics
     df = df.describe()
 
