@@ -108,7 +108,7 @@ def test_register_parquet_partitioned(ctx, tmp_path):
     ctx.register_parquet(
         "datapp",
         str(dir_root),
-        table_partition_cols=["grp"],
+        table_partition_cols=[("grp", "string")],
         parquet_pruning=True,
         file_extension=".parquet",
     )
@@ -145,7 +145,9 @@ def test_execute(ctx, tmp_path):
     assert ctx.tables() == {"t"}
 
     # count
-    result = ctx.sql("SELECT COUNT(a) AS cnt FROM t").collect()
+    result = ctx.sql(
+        "SELECT COUNT(a) AS cnt FROM t WHERE a IS NOT NULL"
+    ).collect()
 
     expected = pa.array([7], pa.int64())
     expected = [pa.RecordBatch.from_arrays([expected], ["cnt"])]
