@@ -20,6 +20,7 @@ use std::path::PathBuf;
 use std::sync::Arc;
 
 use object_store::ObjectStore;
+use url::Url;
 use uuid::Uuid;
 
 use pyo3::exceptions::{PyKeyError, PyValueError};
@@ -35,12 +36,12 @@ use crate::sql::logical::PyLogicalPlan;
 use crate::store::StorageContexts;
 use crate::udaf::PyAggregateUDF;
 use crate::udf::PyScalarUDF;
-use crate::utils::wait_for_future;
+use crate::utils::{get_tokio_runtime, wait_for_future};
 use datafusion::arrow::datatypes::{DataType, Schema};
 use datafusion::arrow::pyarrow::PyArrowType;
 use datafusion::arrow::record_batch::RecordBatch;
-use datafusion::datasource::datasource::TableProvider;
 use datafusion::datasource::MemTable;
+use datafusion::datasource::TableProvider;
 use datafusion::execution::context::{SessionConfig, SessionContext, TaskContext};
 use datafusion::execution::disk_manager::DiskManagerConfig;
 use datafusion::execution::memory_pool::{FairSpillPool, GreedyMemoryPool, UnboundedMemoryPool};
@@ -50,10 +51,11 @@ use datafusion::prelude::{
     AvroReadOptions, CsvReadOptions, DataFrame, NdJsonReadOptions, ParquetReadOptions,
 };
 use datafusion_common::ScalarValue;
-use tokio::runtime::Runtime;
+use pyo3::types::PyTuple;
 use tokio::task::JoinHandle;
 
-#[pyclass(name = "SessionConfig", module = "datafusion", subclass, unsendable)]
+/// Configuration options for a SessionContext
+#[pyclass(name = "SessionConfig", module = "datafusion", subclass)]
 #[derive(Clone, Default)]
 pub(crate) struct PySessionConfig {
     pub(crate) config: SessionConfig,
@@ -139,9 +141,14 @@ impl PySessionConfig {
     fn with_parquet_pruning(&self, enabled: bool) -> Self {
         Self::from(self.config.clone().with_parquet_pruning(enabled))
     }
+
+    fn set(&self, key: &str, value: &str) -> Self {
+        Self::from(self.config.clone().set_str(key, value))
+    }
 }
 
-#[pyclass(name = "RuntimeConfig", module = "datafusion", subclass, unsendable)]
+/// Runtime options for a SessionContext
+#[pyclass(name = "RuntimeConfig", module = "datafusion", subclass)]
 #[derive(Clone)]
 pub(crate) struct PyRuntimeConfig {
     pub(crate) config: RuntimeConfig,
@@ -203,7 +210,7 @@ impl PyRuntimeConfig {
 /// `PySessionContext` is able to plan and execute DataFusion plans.
 /// It has a powerful optimizer, a physical planner for local execution, and a
 /// multi-threaded execution engine to perform the execution.
-#[pyclass(name = "SessionContext", module = "datafusion", subclass, unsendable)]
+#[pyclass(name = "SessionContext", module = "datafusion", subclass)]
 #[derive(Clone)]
 pub(crate) struct PySessionContext {
     pub(crate) ctx: SessionContext,
@@ -217,7 +224,7 @@ impl PySessionContext {
         let config = if let Some(c) = config {
             c.config
         } else {
-            SessionConfig::default()
+            SessionConfig::default().with_information_schema(true)
         };
         let runtime_config = if let Some(c) = runtime {
             c.config
@@ -258,10 +265,9 @@ impl PySessionContext {
         } else {
             &upstream_host
         };
-
-        self.ctx
-            .runtime_env()
-            .register_object_store(scheme, derived_host, store);
+        let url_string = format!("{}{}", scheme, derived_host);
+        let url = Url::parse(&url_string).unwrap();
+        self.ctx.runtime_env().register_object_store(&url, store);
         Ok(())
     }
 
@@ -275,23 +281,29 @@ impl PySessionContext {
     fn create_dataframe(
         &mut self,
         partitions: PyArrowType<Vec<Vec<RecordBatch>>>,
+        name: Option<&str>,
         py: Python,
     ) -> PyResult<PyDataFrame> {
         let schema = partitions.0[0][0].schema();
         let table = MemTable::try_new(schema, partitions.0).map_err(DataFusionError::from)?;
 
-        // generate a random (unique) name for this table
+        // generate a random (unique) name for this table if none is provided
         // table name cannot start with numeric digit
-        let name = "c".to_owned()
-            + Uuid::new_v4()
-                .simple()
-                .encode_lower(&mut Uuid::encode_buffer());
+        let table_name = match name {
+            Some(val) => val.to_owned(),
+            None => {
+                "c".to_owned()
+                    + Uuid::new_v4()
+                        .simple()
+                        .encode_lower(&mut Uuid::encode_buffer())
+            }
+        };
 
         self.ctx
-            .register_table(&*name, Arc::new(table))
+            .register_table(&*table_name, Arc::new(table))
             .map_err(DataFusionError::from)?;
 
-        let table = wait_for_future(py, self._table(&name)).map_err(DataFusionError::from)?;
+        let table = wait_for_future(py, self._table(&table_name)).map_err(DataFusionError::from)?;
 
         let df = PyDataFrame::new(table);
         Ok(df)
@@ -302,6 +314,105 @@ impl PySessionContext {
         PyDataFrame::new(DataFrame::new(self.ctx.state(), plan.plan.as_ref().clone()))
     }
 
+    /// Construct datafusion dataframe from Python list
+    #[allow(clippy::wrong_self_convention)]
+    fn from_pylist(
+        &mut self,
+        data: PyObject,
+        name: Option<&str>,
+        _py: Python,
+    ) -> PyResult<PyDataFrame> {
+        Python::with_gil(|py| {
+            // Instantiate pyarrow Table object & convert to Arrow Table
+            let table_class = py.import("pyarrow")?.getattr("Table")?;
+            let args = PyTuple::new(py, &[data]);
+            let table = table_class.call_method1("from_pylist", args)?.into();
+
+            // Convert Arrow Table to datafusion DataFrame
+            let df = self.from_arrow_table(table, name, py)?;
+            Ok(df)
+        })
+    }
+
+    /// Construct datafusion dataframe from Python dictionary
+    #[allow(clippy::wrong_self_convention)]
+    fn from_pydict(
+        &mut self,
+        data: PyObject,
+        name: Option<&str>,
+        _py: Python,
+    ) -> PyResult<PyDataFrame> {
+        Python::with_gil(|py| {
+            // Instantiate pyarrow Table object & convert to Arrow Table
+            let table_class = py.import("pyarrow")?.getattr("Table")?;
+            let args = PyTuple::new(py, &[data]);
+            let table = table_class.call_method1("from_pydict", args)?.into();
+
+            // Convert Arrow Table to datafusion DataFrame
+            let df = self.from_arrow_table(table, name, py)?;
+            Ok(df)
+        })
+    }
+
+    /// Construct datafusion dataframe from Arrow Table
+    #[allow(clippy::wrong_self_convention)]
+    fn from_arrow_table(
+        &mut self,
+        data: PyObject,
+        name: Option<&str>,
+        _py: Python,
+    ) -> PyResult<PyDataFrame> {
+        Python::with_gil(|py| {
+            // Instantiate pyarrow Table object & convert to batches
+            let table = data.call_method0(py, "to_batches")?;
+
+            // Cast PyObject to RecordBatch type
+            // Because create_dataframe() expects a vector of vectors of record batches
+            // here we need to wrap the vector of record batches in an additional vector
+            let batches = table.extract::<PyArrowType<Vec<RecordBatch>>>(py)?;
+            let list_of_batches = PyArrowType::try_from(vec![batches.0])?;
+            self.create_dataframe(list_of_batches, name, py)
+        })
+    }
+
+    /// Construct datafusion dataframe from pandas
+    #[allow(clippy::wrong_self_convention)]
+    fn from_pandas(
+        &mut self,
+        data: PyObject,
+        name: Option<&str>,
+        _py: Python,
+    ) -> PyResult<PyDataFrame> {
+        Python::with_gil(|py| {
+            // Instantiate pyarrow Table object & convert to Arrow Table
+            let table_class = py.import("pyarrow")?.getattr("Table")?;
+            let args = PyTuple::new(py, &[data]);
+            let table = table_class.call_method1("from_pandas", args)?.into();
+
+            // Convert Arrow Table to datafusion DataFrame
+            let df = self.from_arrow_table(table, name, py)?;
+            Ok(df)
+        })
+    }
+
+    /// Construct datafusion dataframe from polars
+    #[allow(clippy::wrong_self_convention)]
+    fn from_polars(
+        &mut self,
+        data: PyObject,
+        name: Option<&str>,
+        _py: Python,
+    ) -> PyResult<PyDataFrame> {
+        Python::with_gil(|py| {
+            // Convert Polars dataframe to Arrow Table
+            let table = data.call_method0(py, "to_arrow")?;
+
+            // Convert Arrow Table to datafusion DataFrame
+            let df = self.from_arrow_table(table, name, py)?;
+            Ok(df)
+        })
+    }
+
     fn register_table(&mut self, name: &str, table: &PyTable) -> PyResult<()> {
         self.ctx
             .register_table(name, table.table())
@@ -443,8 +554,8 @@ impl PySessionContext {
         Ok(PyDataFrame::new(self.ctx.read_empty()?))
     }
 
-    fn session_id(&self) -> PyResult<String> {
-        Ok(self.ctx.session_id())
+    fn session_id(&self) -> String {
+        self.ctx.session_id()
     }
 
     #[allow(clippy::too_many_arguments)]
@@ -578,11 +689,20 @@ impl PySessionContext {
     }
 
     fn __repr__(&self) -> PyResult<String> {
-        let id = self.session_id();
-        match id {
-            Ok(value) => Ok(format!("SessionContext(session_id={value})")),
-            Err(err) => Ok(format!("Error: {:?}", err.to_string())),
-        }
+        let config = self.ctx.copied_config();
+        let mut config_entries = config
+            .options()
+            .entries()
+            .iter()
+            .filter(|e| e.value.is_some())
+            .map(|e| format!("{} = {}", e.key, e.value.as_ref().unwrap()))
+            .collect::<Vec<_>>();
+        config_entries.sort();
+        Ok(format!(
+            "SessionContext: id={}; configs=[\n\t{}]",
+            self.session_id(),
+            config_entries.join("\n\t")
+        ))
     }
 
     /// Execute a partition of an execution plan and return a stream of record batches
@@ -592,20 +712,13 @@ impl PySessionContext {
         part: usize,
         py: Python,
     ) -> PyResult<PyRecordBatchStream> {
-        let ctx = Arc::new(TaskContext::new(
-            "task_id".to_string(),
-            "session_id".to_string(),
-            HashMap::new(),
-            HashMap::new(),
-            HashMap::new(),
-            Arc::new(RuntimeEnv::default()),
-        ));
+        let ctx: TaskContext = TaskContext::from(&self.ctx.state());
         // create a Tokio runtime to run the async code
-        let rt = Runtime::new().unwrap();
+        let rt = &get_tokio_runtime(py).0;
         let plan = plan.plan.clone();
         let fut: JoinHandle<datafusion_common::Result<SendableRecordBatchStream>> =
-            rt.spawn(async move { plan.execute(part, ctx) });
-        let stream = wait_for_future(py, fut).map_err(|e| py_datafusion_err(e))?;
+            rt.spawn(async move { plan.execute(part, Arc::new(ctx)) });
+        let stream = wait_for_future(py, fut).map_err(py_datafusion_err)?;
         Ok(PyRecordBatchStream::new(stream?))
     }
 }
