@@ -20,7 +20,7 @@ use crate::sql::logical::PyLogicalPlan;
 use crate::utils::wait_for_future;
 use crate::{errors::DataFusionError, expr::PyExpr};
 use datafusion::arrow::datatypes::Schema;
-use datafusion::arrow::pyarrow::{PyArrowConvert, PyArrowException, PyArrowType};
+use datafusion::arrow::pyarrow::{PyArrowType, ToPyArrow};
 use datafusion::arrow::util::pretty;
 use datafusion::dataframe::DataFrame;
 use datafusion::prelude::*;
@@ -76,6 +76,13 @@ impl PyDataFrame {
         }
     }
 
+    /// Calculate summary statistics for a DataFrame
+    fn describe(&self, py: Python) -> PyResult<Self> {
+        let df = self.df.as_ref().clone();
+        let stat_df = wait_for_future(py, df.describe())?;
+        Ok(Self::new(stat_df))
+    }
+
     /// Returns the schema from the logical plan
     fn schema(&self) -> PyArrowType<Schema> {
         PyArrowType(self.df.schema().into())
@@ -165,8 +172,7 @@ impl PyDataFrame {
     #[pyo3(signature = (num=20))]
     fn show(&self, py: Python, num: usize) -> PyResult<()> {
         let df = self.df.as_ref().clone().limit(0, Some(num))?;
-        let batches = wait_for_future(py, df.collect())?;
-        pretty::print_batches(&batches).map_err(|err| PyArrowException::new_err(err.to_string()))
+        print_dataframe(py, df)
     }
 
     /// Filter out duplicate rows
@@ -210,8 +216,7 @@ impl PyDataFrame {
     #[pyo3(signature = (verbose=false, analyze=false))]
     fn explain(&self, py: Python, verbose: bool, analyze: bool) -> PyResult<()> {
         let df = self.df.as_ref().clone().explain(verbose, analyze)?;
-        let batches = wait_for_future(py, df.collect())?;
-        pretty::print_batches(&batches).map_err(|err| PyArrowException::new_err(err.to_string()))
+        print_dataframe(py, df)
     }
 
     /// Get the logical plan for this `DataFrame`
@@ -313,26 +318,89 @@ impl PyDataFrame {
         Ok(())
     }
 
-    /// Convert to pandas dataframe with pyarrow
-    /// Collect the batches, pass to Arrow Table & then convert to Pandas DataFrame
-    fn to_pandas(&self, py: Python) -> PyResult<PyObject> {
-        let batches = self.collect(py);
+    /// Convert to Arrow Table
+    /// Collect the batches and pass to Arrow Table
+    fn to_arrow_table(&self, py: Python) -> PyResult<PyObject> {
+        let batches = self.collect(py)?.to_object(py);
+        let schema: PyObject = self.schema().into_py(py);
 
         Python::with_gil(|py| {
             // Instantiate pyarrow Table object and use its from_batches method
             let table_class = py.import("pyarrow")?.getattr("Table")?;
-            let args = PyTuple::new(py, batches);
+            let args = PyTuple::new(py, &[batches, schema]);
             let table: PyObject = table_class.call_method1("from_batches", args)?.into();
+            Ok(table)
+        })
+    }
 
-            // Use Table.to_pandas() method to convert batches to pandas dataframe
+    /// Convert to pandas dataframe with pyarrow
+    /// Collect the batches, pass to Arrow Table & then convert to Pandas DataFrame
+    fn to_pandas(&self, py: Python) -> PyResult<PyObject> {
+        let table = self.to_arrow_table(py)?;
+
+        Python::with_gil(|py| {
             // See also: https://arrow.apache.org/docs/python/generated/pyarrow.Table.html#pyarrow.Table.to_pandas
             let result = table.call_method0(py, "to_pandas")?;
             Ok(result)
         })
     }
 
+    /// Convert to Python list using pyarrow
+    /// Each list item represents one row encoded as dictionary
+    fn to_pylist(&self, py: Python) -> PyResult<PyObject> {
+        let table = self.to_arrow_table(py)?;
+
+        Python::with_gil(|py| {
+            // See also: https://arrow.apache.org/docs/python/generated/pyarrow.Table.html#pyarrow.Table.to_pylist
+            let result = table.call_method0(py, "to_pylist")?;
+            Ok(result)
+        })
+    }
+
+    /// Convert to Python dictionary using pyarrow
+    /// Each dictionary key is a column and the dictionary value represents the column values
+    fn to_pydict(&self, py: Python) -> PyResult<PyObject> {
+        let table = self.to_arrow_table(py)?;
+
+        Python::with_gil(|py| {
+            // See also: https://arrow.apache.org/docs/python/generated/pyarrow.Table.html#pyarrow.Table.to_pydict
+            let result = table.call_method0(py, "to_pydict")?;
+            Ok(result)
+        })
+    }
+
+    /// Convert to polars dataframe with pyarrow
+    /// Collect the batches, pass to Arrow Table & then convert to polars DataFrame
+    fn to_polars(&self, py: Python) -> PyResult<PyObject> {
+        let table = self.to_arrow_table(py)?;
+
+        Python::with_gil(|py| {
+            let dataframe = py.import("polars")?.getattr("DataFrame")?;
+            let args = PyTuple::new(py, &[table]);
+            let result: PyObject = dataframe.call1(args)?.into();
+            Ok(result)
+        })
+    }
+
     // Executes this DataFrame to get the total number of rows.
     fn count(&self, py: Python) -> PyResult<usize> {
         Ok(wait_for_future(py, self.df.as_ref().clone().count())?)
     }
 }
+
+/// Print DataFrame
+fn print_dataframe(py: Python, df: DataFrame) -> PyResult<()> {
+    // Get string representation of record batches
+    let batches = wait_for_future(py, df.collect())?;
+    let batches_as_string = pretty::pretty_format_batches(&batches);
+    let result = match batches_as_string {
+        Ok(batch) => format!("DataFrame()\n{batch}"),
+        Err(err) => format!("Error: {:?}", err.to_string()),
+    };
+
+    // Import the Python 'builtins' module to access the print function
+    // Note that println! does not print to the Python debug console and is not visible in notebooks for instance
+    let print = py.import("builtins")?.getattr("print")?;
+    print.call1((result,))?;
+    Ok(())
+}
