@@ -21,14 +21,18 @@ use std::fmt::Debug;
 
 use datafusion::arrow::error::ArrowError;
 use datafusion::error::DataFusionError as InnerDataFusionError;
+use prost::EncodeError;
 use pyo3::{exceptions::PyException, PyErr};
 
+pub type Result<T> = std::result::Result<T, DataFusionError>;
+
 #[derive(Debug)]
 pub enum DataFusionError {
     ExecutionError(InnerDataFusionError),
     ArrowError(ArrowError),
     Common(String),
     PythonError(PyErr),
+    EncodeError(EncodeError),
 }
 
 impl fmt::Display for DataFusionError {
@@ -38,6 +42,7 @@ impl fmt::Display for DataFusionError {
             DataFusionError::ArrowError(e) => write!(f, "Arrow error: {e:?}"),
             DataFusionError::PythonError(e) => write!(f, "Python error {e:?}"),
             DataFusionError::Common(e) => write!(f, "{e}"),
+            DataFusionError::EncodeError(e) => write!(f, "Failed to encode substrait plan: {e}"),
         }
     }
 }
@@ -82,3 +87,7 @@ pub fn py_runtime_err(e: impl Debug) -> PyErr {
 pub fn py_datafusion_err(e: impl Debug) -> PyErr {
     PyErr::new::<pyo3::exceptions::PyRuntimeError, _>(format!("{e:?}"))
 }
+
+pub fn py_unsupported_variant_err(e: impl Debug) -> PyErr {
+    PyErr::new::<pyo3::exceptions::PyValueError, _>(format!("{e:?}"))
+}
