@@ -19,6 +19,14 @@
 use mimalloc::MiMalloc;
 use pyo3::prelude::*;
 
+// Re-export Apache Arrow DataFusion dependencies
+pub use datafusion;
+pub use datafusion_common;
+pub use datafusion_expr;
+pub use datafusion_optimizer;
+pub use datafusion_sql;
+pub use datafusion_substrait;
+
 #[allow(clippy::borrow_deref_ref)]
 pub mod catalog;
 pub mod common;
@@ -32,7 +40,7 @@ mod dataset;
 mod dataset_exec;
 pub mod errors;
 #[allow(clippy::borrow_deref_ref)]
-mod expr;
+pub mod expr;
 #[allow(clippy::borrow_deref_ref)]
 mod functions;
 pub mod physical_plan;
@@ -51,12 +59,21 @@ pub mod utils;
 #[global_allocator]
 static GLOBAL: MiMalloc = MiMalloc;
 
+// Used to define Tokio Runtime as a Python module attribute
+#[pyclass]
+pub(crate) struct TokioRuntime(tokio::runtime::Runtime);
+
 /// Low-level DataFusion internal package.
 ///
 /// The higher-level public API is defined in pure python files under the
 /// datafusion directory.
 #[pymodule]
 fn _internal(py: Python, m: &PyModule) -> PyResult<()> {
+    // Register the Tokio Runtime as a module attribute so we can reuse it
+    m.add(
+        "runtime",
+        TokioRuntime(tokio::runtime::Runtime::new().unwrap()),
+    )?;
     // Register the python classes
     m.add_class::<catalog::PyCatalog>()?;
     m.add_class::<catalog::PyDatabase>()?;
