@@ -16,19 +16,26 @@
 // under the License.
 
 use crate::errors::DataFusionError;
+use crate::TokioRuntime;
 use datafusion_expr::Volatility;
 use pyo3::prelude::*;
 use std::future::Future;
 use tokio::runtime::Runtime;
 
+/// Utility to get the Tokio Runtime from Python
+pub(crate) fn get_tokio_runtime(py: Python) -> PyRef<TokioRuntime> {
+    let datafusion = py.import("datafusion._internal").unwrap();
+    datafusion.getattr("runtime").unwrap().extract().unwrap()
+}
+
 /// Utility to collect rust futures with GIL released
 pub fn wait_for_future<F: Future>(py: Python, f: F) -> F::Output
 where
     F: Send,
     F::Output: Send,
 {
-    let rt = Runtime::new().unwrap();
-    py.allow_threads(|| rt.block_on(f))
+    let runtime: &Runtime = &get_tokio_runtime(py).0;
+    py.allow_threads(|| runtime.block_on(f))
 }
 
 pub(crate) fn parse_volatility(value: &str) -> Result<Volatility, DataFusionError> {
@@ -38,9 +45,8 @@ pub(crate) fn parse_volatility(value: &str) -> Result<Volatility, DataFusionErro
         "volatile" => Volatility::Volatile,
         value => {
             return Err(DataFusionError::Common(format!(
-                "Unsupportad volatility type: `{}`, supported \
-                 values are: immutable, stable and volatile.",
-                value
+                "Unsupportad volatility type: `{value}`, supported \
+                 values are: immutable, stable and volatile."
             )))
         }
     })
